package controller.listener;

import java.awt.event.MouseEvent;

import javax.swing.event.MouseInputAdapter;

import controller.DrawableObjectProcessing;

import model.drawables.Circle;
import model.drawables.DashedCircle;
import model.drawables.Point;

/**
 * Ein Listener, der Circles durch MouseEvents erzeugt und an ein Objekt, das
 * DrawableObjectProcessing implementiert übergibt.
 * 
 * @author Nicolas Neubauer
 * 
 */
public class CircleListener extends MouseInputAdapter {

	private Point tmp;
	private DrawableObjectProcessing delegate;

	/**
	 * 
	 * @param delegate
	 */
	public CircleListener(DrawableObjectProcessing delegate) {
		this.delegate = delegate;
	}

	/**
	 * Merke die aktuelle Koordinate des Startpunktes oder, sofern schon
	 * vorhanden, erzeuge mit Hilfe des neuen (End-)punkts einen neuen Circle
	 * und übergebe ihn dem Delegate.
	 * 
	 * @param e
	 */
	public void mouseClicked(MouseEvent e) {

		if (tmp == null)
			tmp = new Point(e.getX(), e.getY());
		else {

			// Lösche temporäres Object
			delegate.clearTemporaryDrawableObject();

			// Berechne Radius
			double dx = tmp.x - e.getX();
			double dy = tmp.y - e.getY();
			int r = (int) Math.sqrt(dx * dx + dy * dy);

			Circle c = new Circle(tmp, r);
			tmp = null;
			delegate.processDrawableObject(c);
		}

	}

	/**
	 * Übergebe mit Hilfe der aktuellen Mausposition einen DashedCircle als
	 * temporäres Objekt an das Delegate, sofern bereits ein Mittelpunkt
	 * vorliegt.
	 * 
	 * @param e
	 */
	public void mouseMoved(MouseEvent e) {
		if (tmp != null) {
			// Berechne Radius
			double dx = tmp.x - e.getX();
			double dy = tmp.y - e.getY();
			int r = (int) Math.sqrt(dx * dx + dy * dy);

			DashedCircle c = new DashedCircle(tmp, r);
			delegate.setTemporaryDrawableObject(c);
		}

	}

}
