/**
 * 
 */
package controller.listener;

import java.awt.event.MouseEvent;
import controller.DrawableObjectProcessing;
import model.drawables.DashedRectangle;
import model.drawables.Point;
import model.drawables.Rectangle;

/**
 * 
 * Ein Listener, der Rectangles durch MouseEvents erzeugt und an ein Objekt, das
 * DrawableObjectProcessing implementiert übergibt.
 * 
 * @author Nicolas Neubauer
 * 
 */
public class ClippingRectangleListener extends RectangleListener {

	private Point tmp;
	private DrawableObjectProcessing delegate;

	/**
	 * Konstruktor
	 * 
	 * @param delegate
	 */
	public ClippingRectangleListener(DrawableObjectProcessing delegate) {
		super(delegate);
		this.delegate = delegate;
	}

	/**
	 * Merke die aktuelle Koordinate des Startpunktes oder, sofern schon
	 * vorhanden, erzeuge mit Hilfe des neuen (End-)punkts ein neues Rectangle
	 * und übergebe es dem Delegate.
	 * 
	 * @param e
	 */
	public void mouseClicked(MouseEvent e) {
		if (tmp == null)
			tmp = new Point(e.getX(), e.getY());
		else {
			// Clip-Rechteck bergeben
			delegate.clipDrawableObjects(new Rectangle(tmp, new Point(e.getX(),
					e.getY())));
			delegate.clearTemporaryDrawableObject();
			tmp = null;
		}
	}

	/**
	 * Übergebe mit Hilfe der aktuellen Mausposition ein DashedRectangle als
	 * temporäres Objekt an das Delegate, sofern bereits ein Mittelpunkt
	 * vorliegt.
	 * 
	 * @param e
	 */
	public void mouseMoved(MouseEvent e) {
		if (tmp != null) {
			// Temporäres Objekt neu zeichnen
			delegate.setTemporaryDrawableObject(new DashedRectangle(tmp,
					new Point(e.getX(), e.getY())));
		}
	}
}
