package model.drawables;

import java.awt.Graphics;

/**
 * Circle Klasse, die einen Kreis mit dem Bresenham-Algorithmus zeichen kann.
 * Ein Kreis besteht aus einem Mittelpunkt und einem Radius.
 * 
 * @author Nicolas Neubauer
 */
public class Circle extends DrawableObject {

	protected Point m; // Mittelpunkt m
	protected int r; // Radius r

	/**
	 * Konstruktor, der einen Kreis aus dem Koordinatenpaar des Mittelpunktes
	 * und dem Radius erstellt.
	 * 
	 * @param x
	 *            X-Koordinate des Mittelpunktes
	 * @param y
	 *            Y-Koordinate des Mittelpunktes
	 * @param r
	 *            Radius des Kreises
	 */
	public Circle(int x, int y, int r) {
		this(new Point(x, y), r);
	}

	/**
	 * Konstruktor, der aus einem Objekt Point und dem Radius einen Kreis
	 * erzeugt.
	 * 
	 * @param m
	 *            Mittelpunkt des Kreises
	 * @param r
	 *            Radius des Kreises
	 */
	public Circle(Point m, int r) {
		this.m = m;
		this.r = r;
	}

	/**
	 * Den Kreis in einen grafischen Kontext zeichnen mit Hilfe des
	 * Bresenham-Algorithmus für Kreise
	 * 
	 * @param g
	 *            der grafische Kontext in den der Kreis gezeichnet werden soll
	 */
	public void paint(Graphics g) {
		int x, y, d, dx, dxy;
		x = 0;
		y = r;
		d = 1 - r;
		dx = 3;
		dxy = -2 * r + 5;
		while (y >= x) {

			// Setze die Pixel fr alle Oktanden
			setPixel(m.x + x, m.y + y, g);
			setPixel(m.x + y, m.y + x, g);
			setPixel(m.x + y, m.y - x, g);
			setPixel(m.x + x, m.y - y, g);
			setPixel(m.x - x, m.y - y, g);
			setPixel(m.x - y, m.y - x, g);
			setPixel(m.x - y, m.y + x, g);
			setPixel(m.x - x, m.y + y, g);

			if (d < 0) {
				d = d + dx;
				dx = dx + 2;
				dxy = dxy + 2;
				x++;
			} else {
				d = d + dxy;
				dx = dx + 2;
				dxy = dxy + 4;
				x++;
				y--;
			}

		}
	}

}
